//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: MeasureReferences.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD function  
//				dscGetReferences(), dscSetReferences(), and dscSetCalMUX().
//              This is sample code describes the usage for Athena-II SBC.
//              Precise voltage sources are output to pins on the board,
//              and the user enters them via the keyboard; these values are
//              later used by the functions dscADAutoCal and dscDAAutoCal,
//              as appropriate to the board.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/27/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif


// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
DSCAUTOCAL dscautocal; // structure containing auto-calibration settings
ERRPARAMS errorParams; // errorParams
int menu;

int i;	     // miscellaneous counter

DFLOAT refs[5];

char input_buffer[20]; // Buffer for parsing user character input.



//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Measure References
//	    IV. Cleanup
//
//=============================================================================

int main( void )
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//=========================================================================

	if( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the AthenaII board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//=========================================================================

	printf ( "\nAthenaII BOARD INITIALIZATION:\n" );

	dsccb.base_address =0x280;	

	dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_ATHENAII, &dsccb, &dscb) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );		
		return 0;
	}

	printf ("\n" );
		

	//=========================================================================
	// III. MEASURE REFERENCES
	//
	//		Measure the reference voltages for your board and then mention how
	//		to store these values in the EEPROM. 
	//
	//=========================================================================

	do {
		printf ("\nSelect from the following menu \n" );
		printf ("0) Exit\n" );
		printf ("1) EEPROM dump \n" );
		printf ("2) Measure A/D reference voltages\n" );

		scanf("%d", &menu);

		if( menu == 0) 
			break;
		else if ( menu == 1 )
		{
			if ( dscGetReferenceVoltages ( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscGetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}
	
			printf ("\nThe stored A/D Reference Voltages are:\n" );
			for( i = 0; i < 4 ; i++)
			{
				 printf ("%d: %9.5lfV\n", i, refs[i] );
			}

			printf ("\n" );

		}
		else if( menu == 2)
		{
		
			printf ( "\nPlace the ground probe of your voltmeter\non pin 2 of J33 and the positive probe \non pin 1 of J33\n\n" );
	
			if ( dscSetCalMux(dscb, 1) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscSetCalMux error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			for( i = 0; i < 4; i++ )
			{
				if( dscADSetChannel(dscb, (unsigned char) i, (unsigned char)i) != DE_NONE )
				{
					dscGetLastError(&errorParams);
					fprintf ( stderr, "dscADSetChannel error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
					return 0;
				}
		
				printf ( "Enter the voltage on the voltmeter (cal mux channel %d): ", i );
				scanf( "%lf", &refs[i] );
			}

			if ( dscSetCalMux(dscb, 0) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscSetCalMux error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			if( dscSetReferenceVoltages( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscSetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
				return 0;
			}

			for( i =0; i < 4; i++)
				refs[i] =0;
	
			if( dscGetReferenceVoltages( dscb, refs) != DE_NONE )
			{
				dscGetLastError(&errorParams);
				fprintf ( stderr, "dscGetReferenceVoltages error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );	
				return 0;
			}

			printf ("The stored A/D Reference Voltages are:\n" );
			for( i = 0; i < 4 ; i++)
			{
				 printf ("%d: %9.5lfV\n", i, refs[i]);
			}

			printf ("\n" );

			printf ( "Complete!\n" );
		}
	} while (1);
	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nMeasureReferences completed. \n" );

	return 0;
} // end main()



#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
