//=============================================================================
// (c) Copyright 2009 Diamond Systems Corporation. Use of this source code
// is subject to the terms of Diamond Systems' Software License Agreement.
// Diamond Systems provides no warranty of proper performance if this
// source code is modified.
//
// File: Watchdog.c   v1.00
//
// Depedency :	Universal Driver version 6.02
//				DSCUD.H - Include file.
//				DSCUDBCL.LIB - Library file for MS-DOS 
//						( USE BC 5.02 compiler )
//				LIBDSCUD_6.02_1.a	- Library file for Linux 2.6.23
//						( USE GCC 3.xx compiler )
//				DSCUD.LIB	- Static library file for Windows XP
//						( USE VC++ 6.0 or Visual Studio 2005 )
//				DSCUD_API.LIB	- Static library for Windows CE 6.0
//						( Use Visual Studio 2005 with Platform Builder 6.0 )
// Description: This sample code demonstrates the usage of UD functions  
//				dscWatchdogTrigger(), dscWatchdogTrigger(), dscWatchdogDisable().
//              This sample code describes the usage for Athena-II SBC. The
//              current configuration of the code demonstrates how to reset
//              the board after the counter finishes.
// ************************* History ********************
// Version		Engineer		Date		Description		
// 1.0			James Moore		07/30/09	Created.
//=============================================================================

#include <stdio.h>

#ifdef _WIN32
#ifndef _WIN32_WCE
#include <conio.h>
#endif

#include <windows.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// DOS
#ifdef __BORLANDC__
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <math.h>
// diamond driver includes
#include "dscud.h"
#endif

// Linux
#if defined(linux)
#include <stdlib.h>
#include <math.h>
#include <time.h>
// diamond driver includes
#include "dscud.h"

#include <sys/time.h>

#endif

// var declarations
BYTE result; // returned error code
DSCB dscb;   // handle used to refer to the board
DSCCB dsccb; // structure containing board settings
ERRPARAMS errorParams;  // structure for returning error code and error string
float floatBuff;        // temp variable of size float
BYTE RebootTimer;  // timer program value

char input_buffer[20]; // Buffer for parsing user character input.

//=============================================================================
// Name: main()
// Desc: Starting function that calls the driver functions used
//
//		 NOTE: By convention, you should capture the BYTE return value for each
//		 driver API call, and check the error code.
//
//     STEPS TO FOLLOW:
//
//	     I. Driver Initialization
//	    II. Board Initialization
//	   III. Watchdog Setup & Run
//	    IV. Cleanup
//
//=============================================================================

int main()
{
	//=========================================================================
	// I. DRIVER INITIALIZATION
	//
	//    Initializes the DSCUD library.
	//
	//    STEPS TO FOLLOW:
	//
	//	  1. initialize the driver, using the driver version for validation
	//=========================================================================

	if ( dscInit( DSC_VERSION ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInit error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// II. BOARD INITIALIZATION
	//
	//	   Initialize the ATHENAII board. This function passes the various
	//	   hardware parameters to the driver and resets the hardware.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. set the board type to DSC_ATHENAII for ATHENAII board
	//	   2. set the base address to 0x280
	//	   3. set the interrupt level 5
	//	   4. intialize and register the board with the driver, after which
	//		  the struct, dscb, now holds the handle for the board
	//=========================================================================

	printf ( "\nATHENAII BOARD INITIALIZATION:\n" );

	dsccb.base_address = 0x280;

	dsccb.int_level = 5;

	if ( dscInitBoard ( DSC_ATHENAII, &dsccb, &dscb ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscInitBoard error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}

	//=========================================================================
	// III. Watchdog Setup & Run
	//
	//	    Setup and enable the ATHENAII Watchdog timer. Once the timer
	//		hits the threshold it will shutdown the ATHENAII
	//
	//	    STEPS TO FOLLOW:
	//
	//	    1. set the 4-bit timer value
	//			NOTE: This timer can achieve a max of 2.1 seconds delay.
	//	    2. set the watchdog timer options
	//			NOTE: The macros for this parameter are in DSCUD.H. Please 
	//			      refer to DSCUD.H for more details.
	//	    4. enable watchdog
	//=========================================================================

	printf ("Enter value of timer (max of 2.175 seconds, minimum of 0.0; default: 2.175): ");
	fgets ( input_buffer, 20, stdin );
	if ( input_buffer[0] == '\n' )
	{
	    	RebootTimer = 15;
	}
	else
	{
	        sscanf ( input_buffer, "%f", &floatBuff );
    		RebootTimer = (floatBuff/2.175f) * 16;  // Timer has a range of 0.0 sec to 2.1 sec,
						     // with a resolution of 2.175 s / 16 or
						     // 145 milli-seconds. 16 is 0xF0.
	}

	if ( dscWatchdogEnable ( dscb, RebootTimer, 0, 0 ) != DE_NONE )
	{
		dscGetLastError(&errorParams);
		fprintf ( stderr, "dscWatchdogEnable error: %s %s\n", dscGetErrorString(errorParams.ErrCode), errorParams.errstring );
		return 0;
	}


	// To disable watchdog uncomment the following code
	//
	// dscWatchdogDisable(dscb);
	//


	// To retrigger watchdog uncomment the following code
	//
	// NOTE: By retriggering the watchdog, all timers on the watchdog is reset
	// to the initial enabled values.
	//
	// dscWatchdogTrigger(dscb);
	//


	//=========================================================================
	// IV. CLEANUP
	//
	//	   Cleanup any remnants left by the program and free the resources used
	//	   by the driver.
	//
	//     STEPS TO FOLLOW:
	//
	//	   1. free the driver resources
	//=========================================================================

	dscFree();

	printf ( "\nWatchdog completed.\n" );

	return 0;
} // end main()


#ifdef _WIN32_WCE
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
	main();

	return 0;
}

#endif
